#!/bin/initsh
#
# /etc/rc.d/rc.S:  System initialization script.
#

# PATH shouldn't be needed anymore as we are using
# hard-coded paths to speed things up.
PATH=/sbin:/usr/sbin:/bin:/usr/bin

# Quietly mount both /proc and /sysfs using '-n'
# since no entry in etc/mtab is possible yet
/sbin/mount -n -v -t proc proc /proc 2> /dev/null
/sbin/mount -n -v -t sysfs sysfs /sys 2> /dev/null

# figure out the kernel version
KVERS=`/bin/uname -r`

# save the kernel command line for use later
#CMDLINE=`/bin/cat /proc/cmdline`
#case $CMDLINE in *'strict'*) STRICT=1 ;; esac

# if the kernel modules are in an image, mount it
if [ -f /lib/modules/$KVERS/kernel-modules-$KVERS.sfs ] ; then
	# this mounts the sfs image *over* the directory where the sfs is located
	mount -n -o loop -t squashfs /lib/modules/$KVERS/kernel-modules-$KVERS.sfs /lib/modules/$KVERS
fi

# Manage /dev entries using udev. udev can be disabled by making the
# /etc/rc.d/rc.udev file non-executable or using "nohotplug" option at boot.
# Need to look into using hotplug2 here instead of udev
if [ -x /etc/rc.d/rc.udev ] ; then
	if /bin/grep -wq sysfs /proc/mounts ; then
		case $CMDLINE in
			*' nohotplug '*) : ;;
			*) action=start . /etc/rc.d/rc.udev ;;
		esac
	fi
fi

# initialize Logical Volume Manager
if [ -x /etc/rc.d/rc.lvm ] ; then
	action=start . /etc/rc.d/rc.lvm
fi
# open any volumes setup by cryptsetup
if [ -x /etc/rc.d/rc.cryptsetup ] ; then
	action=start . /etc/rc.d/rc.cryptsetup
fi

# Enable swapping, ignoring errors from any unavailable swap files
# or better don't even do it just yet -nobody's gonna miss swap here
# /sbin/swapon -a 2> /dev/null

# Start FUSE, if requested:
if [ -x /etc/rc.d/rc.fuse ] ; then
	action=start . /etc/rc.d/rc.fuse
fi

# load any other accessory mount-related stuff here

# Set the system time from the hardware clock using hwclock --hctosys.
# the notickwait option requires patched sources -quieten errors in case
# the patch has not been used
if [ -x /sbin/hwclock ] ; then
	# Check for a broken motherboard RTC clock (where ioports for rtc are
	# unknown) to prevent hwclock causing a hang:
	if ! /bin/grep -q -w rtc0 /proc/ioports ; then
		CLOCK_OPT="--directisa"
	fi
	if /bin/grep -wq "^UTC" /etc/hardwareclock 2> /dev/null ; then
		echo "Setting system time from the hardware clock (UTC)."
		/sbin/hwclock $CLOCK_OPT --utc --hctosys --notickwait 2> /dev/null
	else
		echo "Setting system time from the hardware clock (localtime)."
		/sbin/hwclock $CLOCK_OPT --localtime --hctosys --notickwait 2> /dev/null
	fi
fi

# Test to see if the root partition is read-only, like it ought to be.
READWRITE=no
if /bin/touch /fsrwtestfile 2>/dev/null ; then
	/bin/rm -f /fsrwtestfile
	READWRITE=yes
else
	echo "Testing root filesystem status:  read-only filesystem"
fi

# See if a forced filesystem check was requested at shutdown:
if [ -r /etc/forcefsck ] ; then
	FORCEFSCK="-f"
fi

# Our boot-time has gotten so low that some sata and usb drivers still may have
# not had time to initialize by now -especially  with the 'quiet' kernel option.
# Also, trying to run fsck on the root drive may fail because of slowness of udev
# in creating the root device node
# case $CMDLINE in *' quiet '*) /bin/sleep 2 ;; esac
#

# Check the root filesystem:
if [ ! "$READWRITE" = yes ] ; then
	RETVAL=0
	if [ ! -r /etc/fastboot ] ; then
		echo "Checking root filesystem:"
		/sbin/fsck $FORCEFSCK -C -a /
		RETVAL=$?
	fi
	# An error code of 2 or higher will require a reboot.
	if [ $RETVAL -ge 2 ] ; then
		# An error code equal to or greater than 4 means that some errors
		# could not be corrected.  This requires manual attention, so we
		# offer a chance to try to fix the problem in single-user mode:
		if [ $RETVAL -ge 4 ] ; then
			echo
			echo "***********************************************************"
			echo "*** An error occurred during the root filesystem check. ***"
			echo "*** You will now be given a chance to log into the      ***"
			echo "*** system in single-user mode to fix the problem.      ***"
			echo "***                                                     ***"
			echo "*** If you are using the ext2 filesystem, running       ***"
			echo "*** 'e2fsck -v -y <partition>' might help.              ***"
			echo "***********************************************************"
			echo
			echo "Once you exit the single-user shell, the system will reboot."
			echo
			PS1="(Repair filesystem) \#"; export PS1
			/sbin/sulogin
		else # With an error code of 2 or 3, reboot the machine automatically:
			echo
			echo "***********************************"
			echo "*** The filesystem was changed. ***"
			echo "*** The system will now reboot. ***"
			echo "***********************************"
			echo
		fi
		echo "Unmounting file systems."
		/sbin/umount -a -r
		/sbin/mount -n -o remount,ro /
		echo "Rebooting system."
		/bin/sleep 2
		/sbin/reboot -f
	fi
	# Remount the root filesystem in read-write mode
	echo "Remounting root device with read-write enabled."
	/sbin/mount -w -v -n -o remount /
	if [ $? -gt 0 ] ; then
		echo
		echo "Attempt to remount root device as read-write failed!"
		echo "This is going to cause serious problems."
		echo
		echo "Please press ENTER to continue, then get into"
		/bin/echo -n "your machine and start looking for the problem. " 
		read junk; 
	fi
else
	echo "Testing root filesystem status:  read-write filesystem"
	echo
	echo "*** ERROR: Root partition has already been mounted read-write. Cannot check!"
	echo
	echo "For filesystem checking to work properly, the bootloader"
	echo "must initially mount the root partition as read only."
	echo
	/bin/echo -n "Press ENTER to continue. "
	read junk;
fi
# Done checking root filesystem

# / is now read-write, so files can be written or removed

# Delete any existing mount table
/bin/rm -f /etc/mtab*

# create a new  /etc/mtab, showing the real device path of the '/' partition
realroot=`/bin/readlink /dev/root`
if [ -n $realroot ] ; then
	/bin/grep '^\/dev\/root ' /proc/mounts | /bin/sed "s\/dev\/root \/dev\/${realroot} " > /etc/mtab
else
	# provide an alternative in case /dev/root is not there
	/bin/grep ' / ' /proc/mounts | /bin/grep -v "^rootfs" > /etc/mtab
fi

# add the proc and sysfs entries
/bin/grep -E "^(proc |sysfs )" /proc/mounts >> /etc/mtab


# This loads any kernel modules that are needed.
# Priority is given first to a script named "rc.modules.local", then
# to "rc.modules-$FULL_KERNEL_VERSION", and finally to the plain "rc.modules".
if [ -x /etc/rc.d/rc.modules.local ] && [ -r /proc/modules ] ; then
	echo "Running /etc/rc.d/rc.modules.local:"
	. /etc/rc.d/rc.modules.local
elif [ -x /etc/rc.d/rc.modules-`/bin/uname -r` ] && [ -r /proc/modules ] ; then
	echo "Running /etc/rc.d/rc.modules-`/bin/uname -r`:"
	. /etc/rc.d/rc.modules-`/bin/uname -r`
elif [ -x /etc/rc.d/rc.modules ] && [ -r /proc/modules ] && [ -L /etc/rc.d/rc.modules ] ; then
	echo "Running /etc/rc.d/rc.modules -> `readlink /etc/rc.d/rc.modules`:"
	. /etc/rc.d/rc.modules
elif [ -x /etc/rc.d/rc.modules ] && [ -r /proc/modules ] ; then
	echo "Running /etc/rc.d/rc.modules:"
	. /etc/rc.d/rc.modules
fi

# Configure runtime kernel parameters:
if [ -x /sbin/sysctl ] && [ -r /etc/sysctl.conf ] ; then
	/sbin/sysctl -e -p /etc/sysctl.conf
fi

# Mount usbfs:
if /bin/grep -wq usbfs /proc/filesystems; then
	if ! /bin/grep -wq usbfs /proc/mounts ; then
		if ! /bin/grep -wq usbfs /etc/fstab; then
			/sbin/mount -v -t usbfs usbfs /proc/bus/usb
		else
			/sbin/mount -v /proc/bus/usb
		fi
	fi
fi

# Check all the non-root filesystems:
if [ ! -r /etc/fastboot ] ; then
	echo "Checking non-root filesystems:"
	/sbin/fsck $FORCEFSCK -C -R -A -a
fi

# Mount non-root file systems in fstab, but not NFS or SMB 
# because TCP/IP is not yet configured, and not proc or sysfs
# because those have already been mounted.

echo "Mounting non-root local filesystems:"
# We may need to wait a little bit to let USB and other
# hotplugged devices settle:
# sleep 2

# If udev is used, devpts should already be mounted. If so, don't try 
# to mount it again. By backgrounding, we create a race condition,
# but we'll assume that any essential mounts like /usr and/or /home
# wil be listed 'early' in the fstab
if /bin/grep -wq devpts /proc/mounts ; then
	/sbin/mount -a -v -t nonfs,nosmbfs,nocifs,noproc,nosysfs,nodevpts &
else
	/sbin/mount -a -v -t nonfs,nosmbfs,nocifs,noproc,nosysfs &
fi

# Enable swapping.  This is done late in case a swapfile is used,
# as it can't be enabled until the filesystem it resides on has been
# mounted read-write.
/sbin/swapon -a 2> /dev/null

# attempt to empty write buffers, which makes sure all mounts are up
/bin/sync

# if the kernel sources are in an image, mount it
# we've waited till now because /usr/src is commonly mounted from a separate partiton
if [ -f /usr/src/linux-$KVERS/kernel-source-kiss-$KVERS.sfs ] ; then
	# this mounts the sfs image *over* the directory where the sfs is located
	mount -o loop -t squashfs /usr/src/linux-$KVERS/kernel-source-kiss-$KVERS.sfs /usr/src/linux-$KVERS
fi

# This would be a good point to use preloading libraries
# or starting GUI processes early. But, the gains from
# doing so are not as big as one would hope.

# Clean up some temporary files in a separate, backgrounded process
# use /bin/ls instead of globbing
/bin/rm -f `/bin/ls  /var/run/* /var/run/*/* \
	/var/run/*/*/* /etc/nologin /etc/dhcpc/*.pid \
	/etc/forcefsck /etc/fastboot /tmp/.X*lock \
	/var/state/saslauthd/saslauthd.pid /tmp/.Xauth* \
	/var/lock/* /var/spool/uucp/LCK..* /tmp/core /core \
	/var/run/*.pid /tmp/hunt* 2> /dev/null` 2> /dev/null
	
/bin/rm -rf `/bin/ls  /var/lib/tpkg/setup/tmp/* \
	/tmp/kde-[a-zA-Z]* /tmp/ksocket-[a-zA-Z]* \
	/tmp/hsperfdata_[a-zA-Z]* /tmp/plugtmp* 2> /dev/null` 2> /dev/null
# for the moment we clean up /var/tmp
# /bin/rm -rf /var/tmp/* 2> /dev/null

# make sure these directories exist with proper permissions
/bin/mkdir -p /tmp/.ICE-unix 2> /dev/null
/bin/chmod 1777 /tmp/.ICE-unix 2> /dev/null
/bin/mkdir -p /tmp/.X11-unix 2> /dev/null
/bin/chmod 1777 /tmp/.X11-unix 2> /dev/null

# ensure sane permissions on a few dirs
/bin/chmod 755 / /var
/bin/chmod 1777 /tmp /var/lock /var/tmp /var/spool/mail
/bin/chmod 775 /var/run

# Update the current kernel level in the /etc/motd (Message Of The Day) file,
# if the first line of that file begins with the word 'Linux'.
# You are free to modify the rest of the file as you see fit.
if [ -x /bin/sed ] ; then
	/bin/sed -i "{1s/^Linux.*/`/bin/uname -sr`\./}" /etc/motd
fi

# If there are SystemV init scripts for this runlevel, run them.
if [ -x /etc/rc.d/rc.sysvinit ] ; then
	. /etc/rc.d/rc.sysvinit
fi

# Run serial port setup script:
# CAREFUL!  This can make some systems hang if the rc.serial script isn't
# set up correctly.  If this happens, you may have to edit the file from a
# boot disk, and/or set it as non-executable:
if [ -x /etc/rc.d/rc.serial ] ; then
	( action=start /etc/rc.d/rc.serial start )
fi

# Carry an entropy pool between reboots to improve randomness.
# Do it in a backgrounded process
( if [ -f /etc/random-seed ] ; then
	echo "Using /etc/random-seed to initialize /dev/urandom."
	/bin/cat /etc/random-seed > /dev/urandom
fi
# Use the pool size from /proc, or 512 bytes:
if [ -r /proc/sys/kernel/random/poolsize ] ; then
	/bin/dd if=/dev/urandom of=/etc/random-seed count=1 bs=`/bin/cat /proc/sys/kernel/random/poolsize` 2> /dev/null
else
	/bin/dd if=/dev/urandom of=/etc/random-seed count=1 bs=512 2> /dev/null
fi
/bin/chmod 600 /etc/random-seed
) &

# Create a fresh utmp file:
/bin/touch /var/run/utmp
/bin/chown root:utmp /var/run/utmp
/bin/chmod 664 /var/run/utmp
